@@ -294,6 +294,12 @@ void p5_sv_refcnt_inc(PerlInterpreter *my_perl, SV *sv) {
     SvREFCNT_inc(sv);
 }
 
+void p5_sv_destroy(PerlInterpreter *my_perl, SV *sv) {
+    PERL_SET_CONTEXT(my_perl);
+    if (!PL_in_clean_objs && !PL_in_clean_all)
+        SvREFCNT_dec(sv); // don't bother during global deconstruction
+}
+
 void p5_sv_2mortal(PerlInterpreter *my_perl, SV *sv) {
     sv_2mortal(sv);
 }
@@ -596,7 +602,7 @@ SV *p5_call_package_method(PerlInterpreter *my_perl, char *package, char *name,
         SAVETMPS;
 
         PUSHMARK(SP);
-        XPUSHs(newSVpv(package, 0)); // FIXME doesn't this leak the package name?!
+        XPUSHs(sv_2mortal(newSVpv(package, 0)));
         push_arguments(sp, len, args);
 
         *count = call_method(name, flags);
@@ -1199,10 +1205,6 @@ void p5_rebless_object(PerlInterpreter *my_perl, SV *obj, char *package, IV i) {
     }
 }
 
-void p5_remove_magic(PerlInterpreter *my_perl, SV *obj) {
-    mg_free_type(obj, PERL_MAGIC_ext);
-}
-
 SV *p5_add_magic(PerlInterpreter *my_perl, SV *inst, IV i) {
     PERL_SET_CONTEXT(my_perl);
     {
@@ -1496,7 +1498,7 @@ XS(p5_destroy_p5_object) {
         }
     }
     sp -= items;
-    XPUSHs(newSViv(destroyed));
+    XPUSHs(newSViv(destroyed && !PL_in_clean_objs));
     XSRETURN(1);
 }
 
